# coding: utf-8
#
# barbacompta - accounting for dummies
# Copyright (C) 2010-2019 Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

from decimal import Decimal

from django import forms
from django.db.transaction import atomic
from django.core.exceptions import ValidationError
from django.contrib.admin import widgets

from . import models

pourcentages = [(Decimal(i) / 100, "%s %%" % i) for i in range(0, 101, 5)]


class RapidFactureForm(forms.Form):
    contrat = forms.ModelChoiceField(queryset=models.Contrat.objects.all())
    pourcentage = forms.ChoiceField(choices=pourcentages, initial='1')
    solde = forms.BooleanField(
        required=False, label='Ignorer le pourcentage, et solder le contrat en une ligne.'
    )

    def __init__(self, request=None, *args, **kwargs):
        self.request = request
        super(RapidFactureForm, self).__init__(*args, **kwargs)

    @atomic
    def clean(self):
        contrat = self.cleaned_data['contrat']
        pourcentage = Decimal(self.cleaned_data['pourcentage'])
        solde = self.cleaned_data['solde']
        facture = models.Facture(contrat=contrat, creator=self.request.user)
        try:
            facture.clean()
        except ValidationError as e:
            raise forms.ValidationError(*e.args)
        facture.save()
        self.cleaned_data["facture"] = facture
        lignes = []
        errors = []
        if solde:
            montant_solde = contrat.solde()
            if montant_solde == 0:
                raise ValidationError('Le solde du contrat est déjà nul.')
            models.Ligne.objects.create(
                facture=facture, intitule='Solde', prix_unitaire_ht=montant_solde, quantite=Decimal(1)
            )
        else:
            for prestation in contrat.prestations.all():
                ligne = models.Ligne(
                    facture=facture,
                    intitule=prestation.intitule,
                    prix_unitaire_ht=prestation.prix_unitaire_ht,
                    quantite=prestation.quantite * pourcentage,
                )
                lignes.append(ligne)
                try:
                    ligne.clean()
                except ValidationError as e:
                    error = u"Il y a un problème avec la ligne « %s »: " % prestation.intitule
                    error += "; ".join(map(lambda x: x.rstrip("."), e.messages))
                    errors.append(error)
        if errors:
            raise forms.ValidationError(errors)
        else:
            for ligne in lignes:
                ligne.save()
        return self.cleaned_data


class DuplicateContractForm(forms.Form):
    contrat = forms.ModelChoiceField(queryset=models.Contrat.objects.all())
    new_intitule = forms.CharField(max_length=150, label=u"Nouvel intitulé")

    def __init__(self, request=None, *args, **kwargs):
        self.request = request
        super(DuplicateContractForm, self).__init__(*args, **kwargs)

    @atomic
    def clean(self):
        contrat = self.cleaned_data['contrat']
        new_intitule = self.cleaned_data['new_intitule']
        new_contrat = models.Contrat(
            client=contrat.client,
            intitule=new_intitule,
            description=contrat.description,
            tva=contrat.tva,
            creator=self.request.user,
        )
        try:
            new_contrat.clean()
        except ValidationError as e:
            raise forms.ValidationError(*e.args)
        new_contrat.save()
        self.cleaned_data["new_contrat"] = new_contrat
        for prestation in contrat.prestations.all():
            new_prestation = prestation.duplicate()
            new_prestation.contrat = new_contrat
            new_prestation.save()
        return self.cleaned_data


class LigneForm(forms.ModelForm):
    class Meta:
        exclude = ()
        model = models.Ligne
        localized_fields = ("quantite", "prix_unitaire_ht", "taux_tva")


class PrestationForm(forms.ModelForm):
    class Meta:
        exclude = ()
        model = models.Prestation
        localized_fields = ("quantite", "prix_unitaire_ht")


class FactureForm(forms.ModelForm):
    class Meta:
        exclude = ()
        model = models.Facture
        localized_fields = ("taux_tva",)


class ClientForm(forms.ModelForm):
    class Meta:
        exclude = ()
        model = models.Client
        localized_fields = ("tva",)
        widgets = {
            "adresse": widgets.AdminTextareaWidget(attrs={"rows": 4}),
            "contacts": widgets.AdminTextareaWidget(attrs={"rows": 4}),
        }


class ContratForm(forms.ModelForm):
    class Meta:
        exclude = ()
        model = models.Contrat
        localized_fields = ("tva", "montant_sous_traite")


class PaymentForm(forms.ModelForm):
    class Meta:
        exclude = ()
        model = models.Payment
        localized_fields = ("montant_affecte",)
